@@ -87,6 +87,32 @@ static const struct poptOption options[] = {
 	{ NULL } 
 };
 
+void
+buddy_error (GtkWidget *parent, const char *msg, ...)
+{
+	GtkWidget *w;
+	GtkDialog *d;
+	gchar *s;
+	va_list args;
+
+	/* No va_list version of dialog_new, construct the string ourselves. */
+	va_start (args, msg);
+	s = g_strdup_vprintf (msg, args);
+	va_end (args);
+
+	w = gtk_message_dialog_new (GTK_WINDOW (parent),
+				    0,
+				    GTK_MESSAGE_ERROR,
+				    GTK_BUTTONS_OK,
+				    "%s",
+				    s);
+	d = GTK_DIALOG (w);
+	gtk_dialog_set_default_response (d, GTK_RESPONSE_OK);
+	gtk_dialog_run (d);
+	gtk_widget_destroy (w);
+	g_free (s);
+}
+
 GtkWidget *
 get_widget (const char *name, const char *loc)
 {
@@ -190,7 +216,6 @@ update_crash_type (GtkWidget *w, gpointer data)
 void
 on_gdb_go_clicked (GtkWidget *w, gpointer data)
 {
-	druid_data.explicit_dirty = TRUE;
 	start_gdb ();
 }
 
@@ -220,7 +245,6 @@ on_gdb_stop_clicked (GtkWidget *button, gpointer data)
 			return;
 		}
 		stop_gdb ();
-		druid_data.explicit_dirty = TRUE;
 	}
 	gtk_widget_destroy (w);
 }
@@ -265,18 +289,11 @@ on_gdb_save_clicked (GtkWidget *w, gpointer data)
 					      _("Please wait while Bug Buddy saves the stack trace..."),
 					      file, text, -1, &gerr)) {
 			if (gerr) {
-				GtkWidget *d;
-				
-				d = gtk_message_dialog_new (GTK_WINDOW (filechooser),
-							    0,
-							    GTK_MESSAGE_ERROR,
-							    GTK_BUTTONS_OK,
-							    _("The stack trace was not saved in %s:\n\n"
-							      "%s\n\n"
-							      "Please try again, maybe with a different file name."),
-							    file, gerr->message);
-				gtk_dialog_run (GTK_DIALOG (d));
-				gtk_widget_destroy (d);
+				buddy_error (filechooser,
+					     _("The stack trace was not saved in %s:\n\n"
+					       "%s\n\n"
+					       "Please try again, maybe with a different file name."),
+					     file, gerr->message);
 				g_error_free (gerr);
 			}
 			g_free (file);
@@ -310,9 +327,6 @@ on_list_button_press_event (GtkWidget *w, GdkEventButton *button, gpointer data)
 	if (button->type != GDK_2BUTTON_PRESS)
 		return;
 
-	if (druid_data.download_in_progress)
-		return;
-	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (w));
 	if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
 		return;
@@ -490,6 +504,7 @@ init_ui (void)
 	const char **sp;
 	int i;
 	const char *nbs[] = { "druid-notebook", "gdb-notebook", NULL };
+	gboolean have_appname = FALSE;
 	
 	glade_xml_signal_autoconnect (druid_data.xml);
 
@@ -508,17 +523,19 @@ init_ui (void)
 		s = getenv ("GNOME_CRASHED_APPNAME");
 		if (s)
 			g_warning (_("$GNOME_CRASHED_APPNAME is deprecated.\n"
-				     "Please use the --appname command line"
+				     "Please use the --appname command line "
 				     "argument instead."));
 	}
 	buddy_set_text ("gdb-binary-entry", s);
+	if (s)
+		have_appname = TRUE;
 
 	s = popt_data.pid;
 	if (!s) {
 		s = getenv ("GNOME_CRASHED_PID");
 		if (s)
 			g_warning (_("$GNOME_CRASHED_PID is deprecated.\n"
-				     "Please use the --pid command line"
+				     "Please use the --pid command line "
 				     "argument instead."));
 	}
 	if (s) {
@@ -526,6 +543,9 @@ init_ui (void)
 		druid_data.bug_type = BUG_CRASH;
 		druid_data.crash_type = CRASH_DIALOG;
 		druid_set_state (STATE_GDB);
+		if (!have_appname)
+			g_warning (_("To debug a process, the application name is also required.\n"
+				     "Please also supply an --appname command line argument."));
 	}
 
 	/* core crash page */
@@ -628,7 +648,6 @@ on_druid_window_style_set (GtkWidget *widget, GtkStyle *old_style, gpointer data
 int
 main (int argc, char *argv[])
 {
-	GtkWidget *w;
 	GtkIconInfo *icon_info;
 	char *s;
 		
@@ -664,18 +683,10 @@ main (int argc, char *argv[])
 		druid_data.xml = NULL;
 
 	if (!druid_data.xml) {
-		w = gtk_message_dialog_new (NULL,
-					    0,
-					    GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_OK,
-					    _("Bug Buddy could not load its user interface file (%s).\n"
-					      "Please make sure Bug Buddy was installed correctly."), 
-					    s);
-					      
-		gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
+		buddy_error (NULL, 
+			     _("Bug Buddy could not load its user interface file (%s).\n"
+			       "Please make sure Bug Buddy was installed correctly."), 
+			     s);
 		return 0;
 	}
 	g_free (s);	
@@ -695,20 +706,11 @@ main (int argc, char *argv[])
 			if (response == GTK_RESPONSE_CANCEL) {
 				gnome_vfs_async_cancel (druid_data.vfshandle);
 				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
-				druid_data.download_in_progress = FALSE;
 			} else if (response == DOWNLOAD_FINISHED_ZERO_RESPONSE) {
 				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
-				w = gtk_message_dialog_new (NULL,
-							    0,
-							    GTK_MESSAGE_ERROR,
-							    GTK_BUTTONS_OK,
-							    _("Bug Buddy could not update its bug information.\n"
-							      "The old one will be used."));
-					      
-				gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 		GTK_RESPONSE_OK);
-				gtk_dialog_run (GTK_DIALOG (w));
-				gtk_widget_destroy (w);
+				buddy_error (NULL,
+					     _("Bug Buddy could not update its bug information.\n"
+					       "The old one will be used."));
 			} else if (response == DOWNLOAD_FINISHED_OK_RESPONSE) {
 				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
 			}
@@ -722,7 +724,11 @@ main (int argc, char *argv[])
 	gtk_widget_show (GET_WIDGET ("druid-window"));
 
 	load_applications ();
-	start_gdb ();
+
+	if (druid_data.bug_type == BUG_CRASH)
+		start_gdb ();
+
+	products_list_load ();
 
 	gtk_main ();
 
