@@ -286,9 +286,10 @@ async_update (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressInfo *info, gpoin
 	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
 		d(g_print ("Transfer finished\n"));
 		end_bugzilla_download (count);
+		return FALSE;
 	}
 
-	return druid_data.download_in_progress;
+	return TRUE;
 }
 
 /**
@@ -535,8 +536,13 @@ load_bugzilla (const char *filename)
 	bts->severity_node   = gnome_config_get_string ("severity_node=severities");
 	bts->severity_item   = gnome_config_get_string ("severity_item=severity");
 	bts->severity_header = gnome_config_get_string ("severity_header=Severity");
-	if ((time (NULL) - druid_data.last_update_check) > CHECK_UPDATES_TIME )
-		bugzilla_get_md5sums (bts, gnome_config_get_string ("md5sums"));
+	if ((time (NULL) - druid_data.last_update_check) > CHECK_UPDATES_TIME ) {
+		s = gnome_config_get_string ("md5sums");
+		if (s && s!=NULL) {
+			bugzilla_get_md5sums (bts, s);
+			g_free (s);
+		}
+	}
 	
 	bts->products_xml = get_xml_file (bts, "products", 
 					  bts->products_xml_md5,
@@ -630,9 +636,6 @@ load_bugzilla_xml (void)
 	d(g_print ("loading xml..\n"));
 
 	g_hash_table_foreach (druid_data.bugzillas, (GHFunc)load_bugzilla_xml_cb, NULL);
-
-	if (druid_data.show_products)
-		products_list_load ();
 }
 
 static void
@@ -855,7 +858,6 @@ create_components_list (void)
 void
 end_bugzilla_download (int count)
 {
-	druid_data.download_in_progress = FALSE;
 	if (count>0)
 		gtk_dialog_response (GTK_DIALOG (GET_WIDGET ("progress-dialog")),
 				     DOWNLOAD_FINISHED_OK_RESPONSE);
@@ -868,7 +870,6 @@ end_bugzilla_download (int count)
 gboolean
 start_bugzilla_download (void)
 {
-	druid_data.download_in_progress = TRUE;
 	gnome_vfs_async_xfer (	    
 		&druid_data.vfshandle,
 		druid_data.dlsources,
@@ -887,26 +888,18 @@ void
 load_bugzillas (void)
 {
 	BugzillaBTS *bts;
-	GtkWidget *w;
 	DIR *dir;
 	struct dirent *dent;
 	char *p;
 
 	dir = opendir (BUDDY_DATADIR"/bugzilla/");
 	if (!dir) {
-		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
-					    0,
-					    GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_OK,
-					    _("Bug Buddy could not open '%s'.\n"
-					      "Please make sure Bug Buddy was "
-					      "installed correctly.\n\n"
-					      "Bug Buddy will now quit."),
-					    BUDDY_DATADIR "/bugzilla/");
-		gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
+		buddy_error (GTK_WIDGET ("druid-window"), 
+			     _("Bug Buddy could not open '%s'.\n"
+			       "Please make sure Bug Buddy was "
+			       "installed correctly.\n\n"
+			       "Bug Buddy will now quit."),
+			     BUDDY_DATADIR "/bugzilla/");
 		exit (0);
 	}
 
@@ -939,19 +932,12 @@ load_bugzillas (void)
 	closedir (dir);
 
 	if (druid_data.need_to_download && !druid_data.dlsources) {
-		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
-					    0,
-					    GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_OK,
-					    _("Bug Buddy could not find any information on "
-					      "where to submit bugs.\n\n"
-					      "Please make sure Bug Buddy was "
-					      "installed correctly.\n\n"
-					      "Bug Buddy will now quit."));
-		gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
+		buddy_error (GET_WIDGET ("druid-window"),
+			     _("Bug Buddy could not find any information on "
+			       "where to submit bugs.\n\n"
+			       "Please make sure Bug Buddy was "
+			       "installed correctly.\n\n"
+			       "Bug Buddy will now quit."));
 		exit (0);
 	}
 
@@ -1342,8 +1328,6 @@ products_list_load (void)
 	druid_data.product = NULL;
 
 	if (druid_data.show_products) {
-		if (druid_data.download_in_progress)
-			return;
 		g_hash_table_foreach (druid_data.bugzillas, (GHFunc)add_products, store);
 	} else {
 		g_slist_foreach (druid_data.applications, (GFunc)add_application, store);
@@ -1488,7 +1472,7 @@ generate_email_text (gboolean include_headers)
 	email = g_string_new (NULL);
 
 	subject    = buddy_get_text ("desc-subject");
-	version    = druid_data.version ? druid_data.version : strdup("unspecified");
+	version    = druid_data.version ? druid_data.version : "unspecified";
 
 	s          = buddy_get_text ("desc-text");
 	body = format_for_width (s);
@@ -1547,7 +1531,6 @@ generate_email_text (gboolean include_headers)
 		g_string_append_printf (email, "\n\nDebugging Information:\n\n%s\n", debug_info);
 
 	g_free (subject);
-	g_free (version);
 	g_free (body);
 	g_free (debug_info);
 
